import xpcom, xpcom.server
import hotshot, threading

class koProfile:
    def __init__(self):
        self.prof = hotshot.Profile("kogrind.prof", lineevents=1)
        self.lock = threading.Lock()
        
    def __del__(self):
        self.prof.close()
        
    def acquire(self):
        return self.lock.acquire(0)
        
    def release(self):
        self.lock.release()

# store in xpcom module
xpcom._koprofiler = koProfile()

getters = {}
setters = {}

# A wrapper around a function - looks like a function,
# but actually profiles the delegate.
class TracerDelegate:
    def __init__(self, callme):
        self.callme = callme
    def __call__(self, *args):
        if not xpcom._koprofiler.acquire():
            return apply(self.callme, args)
        try:
            return xpcom._koprofiler.prof.runcall(self.callme, *args)
        finally:
            xpcom._koprofiler.release()

# A wrapper around each of our XPCOM objects.  All PyXPCOM calls
# in are made on this object, which creates a TracerDelagate around
# every function.  As the function is called, it collects profile info.
class Tracer:
    def __init__(self, ob):
        self.__dict__['_ob'] = ob
    def __repr__(self):
        return "<Tracer around %r>" % (self._ob,)
    def __str__(self):
        return "<Tracer around %r>" % (self._ob,)
    def __getattr__(self, attr):
        ret = getattr(self._ob, attr) # Attribute error just goes up
        if callable(ret):
            return TracerDelegate(ret)
        else:
            if not attr.startswith("_com_") and not attr.startswith("_reg_"):
                getters[attr] = getters.setdefault(attr,0) + 1
            return ret
    def __setattr__(self, attr, val):
        if self.__dict__.has_key(attr):
            self.__dict__[attr] = val
            return
        setters[attr] = setters.setdefault(attr,0) + 1
        setattr(self._ob, attr, val)

# Installed as a global XPCOM function that if exists, will be called
# to wrap each XPCOM object created.
def MakeTracer(ob):
    # In some cases we may be asked to wrap ourself, so handle that.
    if isinstance(ob, Tracer):
        return ob
    return Tracer(ob)

def UnwrapTracer(ob):
    if isinstance(ob, Tracer):
        return ob._ob
    return ob

xpcom.server.tracer = MakeTracer
xpcom.server.tracer_unwrap = UnwrapTracer
