/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Komodo startup/shutdown handling.
 *
 * This file should ONLY contain functionality directly relevant to startup and
 * shutdown. Enforcer: ShaneC.
 */

var gPrefSvc = null;
var gPrefs = null;
var gInfoSvc = null;
var _gCommandmentSvc = null;

// XXX don't use this for new code, it's here to support
// older code
var gObserverSvc = null;

function KomodoPrefsInit() {
    gPrefSvc = Components.classes["@activestate.com/koPrefService;1"].
                    getService(Components.interfaces.koIPrefService);
    gPrefs = gPrefSvc.prefs;
    KomodoRegisterOnUnloadHandler(KomodoPrefsOnClose);
}

function KomodoPrefsOnClose() {
    gPrefSvc.shutDown();
}

function _komodoInitTemplateService(type) {
    var templateSvc = Components.classes["@activestate.com/koTemplateService?type="+type+";1"].getService();
    try {
        templateSvc.initializeUserTemplateTree();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                            getService(Components.interfaces.koILastErrorService);
        var errmsg = lastErrorSvc.getLastErrorMessage();
        alert("There was an error initializing your Komodo user "+
                     "settings directory with "+type+" template information: "+errmsg+
                     ". This may mean that you will not be able to create "+
                     "your own custom "+type+" templates. You will still be able "+
                     "to use Komodo's numerous standard "+type+" templates.");
    }
}

function KomodoServicesInit()
{
    gPrefSvc.getPrefs("viewStateMRU").max_length = gPrefs.getLongPref("viewStateMRUSize");
    gPrefSvc.getPrefs("docStateMRU").max_length = gPrefs.getLongPref("docStateMRUSize");

    _komodoInitTemplateService("file");
    _komodoInitTemplateService("project");

    var featureStatusSvc = Components.classes["@activestate.com/koFeatureStatusService;1"].
            getService(Components.interfaces.koIFeatureStatusService);
    featureStatusSvc.initialize();

    gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
            getService(Components.interfaces.nsIObserverService);
    KomodoRegisterOnUnloadHandler(KomodoServicesShutdown);
}

function KomodoServicesShutdown()
{
    var featureStatusSvc = Components.classes["@activestate.com/koFeatureStatusService;1"].
            getService(Components.interfaces.koIFeatureStatusService);
    featureStatusSvc.finalize();

    var docSvc = Components.classes["@activestate.com/koDocumentService;1"].
                 getService(Components.interfaces.koIDocumentService);
    docSvc.unload(); // shut down the document service

    // Save the view MRU (*sob* - this needs more thought -
    // really want to save all pref files known to the service at shutdown.
    gPrefSvc.savePrefsState("viewStateMRU");
}



//
// This observer handles any notification pertinent
// to komodo the application (ie. startup/shutdown issues)
// that have no other place to be handled.
//
var _gKomodoObserver = null;
function _KomodoObserver() {
    KomodoRegisterOnUnloadHandler(this.shutdown, this);
    this.startup();
};
_KomodoObserver.prototype.constructor = _KomodoObserver;
_KomodoObserver.prototype.QueryInterface = function (iid) {
    if (!iid.equals(Components.interfaces.nsIObserver) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
}
_KomodoObserver.prototype.startup = function()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    observerSvc.addObserver(this, "quit",false);
    observerSvc.addObserver(this, "open-url",false);
}
_KomodoObserver.prototype.shutdown = function()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    try {
        observerSvc.removeObserver(this, "quit");
        observerSvc.removeObserver(this, "open-url");
    } catch(e) { /* moz already removed them */ log.debug('quit '+e); }
    _gKomodoObserver = null;
}
_KomodoObserver.prototype.observe = function(subject, topic, data)
{
    //log.debug("_KomodoObserver: observed '"+topic+"' notification: data='"+data+"'\n");
    switch (topic) {
    case 'quit':
        window.setTimeout("KomodoGoQuitApplication()", 0);
        break;
    case 'open-url': // see nsCommandLineServiceMac.cpp, bug 37787
        // This is also used by komodo macro API to open files from python
        var urllist = data.split('|'); //see asCommandLineHandler.js
        for (var i in urllist) {
            open_openURI(urllist[i]);
        }
        break;
    }
}




























































































































function KomodoOnLoad(event) {
    // XXX PLUGINS cannot be touched here, do it in the delayed onload handler below!!!
    try {
        trace_onload();






















        KomodoPrefsInit();

        // Setup Komodo default file encoding.
        // Komodo can only handle _some_ encodings out there. Typically
        // Komodo will use the current system encoding (as returned by
        // koInitService.getStartupEncoding()) as the default file encoding.
        // However if the system encoding is one that Komodo cannot handle
        // then we fallback to an encoding that we can handle.
        var initSvc = Components.classes["@activestate.com/koInitService;1"].
                      getService(Components.interfaces.koIInitService);
        var encodingSvc = Components.classes["@activestate.com/koEncodingServices;1"].
                          getService(Components.interfaces.koIEncodingServices);
        // Determine the currently configured default file encoding.
        var useSystemEncoding = gPrefs.getBooleanPref("encodingEnvironment");
        var defaultEncoding = (useSystemEncoding ?
                               initSvc.getStartupEncoding() :
                               gPrefs.getStringPref("encodingDefault"));
        log.debug("encoding: currently configured default is '"+
                  defaultEncoding+"'");
        // Ensure the default encoding can be handled by Komodo.
        if (encodingSvc.get_encoding_index(defaultEncoding) == -1) {
            // The current default encoding is NOT supported.
            log.debug("encoding: '"+defaultEncoding+"' is not supported");
            defaultEncoding = null;
            if (useSystemEncoding) {
                defaultEncoding = gPrefs.getStringPref("encodingDefault");
                log.debug("encoding: try to fallback to 'encodingDefault' "+
                          "pref setting: '"+defaultEncoding+"'");
                if (encodingSvc.get_encoding_index(defaultEncoding) == -1) {
                    // the default encoding in prefs is no good either
                    log.debug("encoding: '"+defaultEncoding+
                              "' is not supported either");
                    defaultEncoding = null;
                }
            }
            if (! defaultEncoding) {
                // Western European is our last resort fallback.
                defaultEncoding = "iso8859-1";
                log.debug("encoding: fallback to '"+defaultEncoding
                          +"' (Western European)");
            }
            gPrefs.setBooleanPref("encodingEnvironment", false);
        }
        //XXX Komodo code requires the encodingDefault string to be lowercase
        //    and while Komodo code has been updated to guarantee this there
        //    may still be uppercase user prefs out there.
        defaultEncoding = defaultEncoding.toLowerCase();
        //XXX Unfortunately we have to write the default encoding to user
        //    prefs even if the system encoding is being used because
        //    most Komodo code using "encodingDefault" does not honour
        //    "encodingEnvironment".
        gPrefs.setStringPref("encodingDefault", defaultEncoding);

        // simple globals that do not need a shutdown
        gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].
              getService(Components.interfaces.koIInfoService);
        
        /* services needed for even the most basic operation of komodo */
        windowManager_init()
        keybindingManagerOnLoad();





        /* setup observers */
        _gKomodoObserver = new _KomodoObserver();




        KomodoServicesInit();
        MRU_initialize();

        views_onload();
        command_onload();



        findtoolbar_onload();
        StatusBar.Init();
        isearch_onload();
        projectManagerOnLoad();

        FindResultsTab_OnLoad();
        Toolbox_onLoad();
        uilayout_onload();
        lint_initialize();



        window.controllers.appendController(new command_broadcasterController());
        // anything that we want to do user interaction with at
        // startup should go into this timeout to let the window
        // onload handler finish.
        window.setTimeout("KomodoOnLoadDelay();",500);
    } catch (e) {
        log.exception(e,"Error doing KomodoOnLoad:");
        throw e;
    }
}

/* generaly this holds anything that requires user interaction
   on startup.  We want that stuff to happen after komodo's main
   windows is up and running.  There may be a thing or two otherwise
   that also needs to start late. */
function KomodoOnLoadDelay() {
    try {
        // Used by perf_timeline.perf_startup. Mark before
        // commandmentSvc.initialize() because that will immediately start
        // executing queued up commandments.
        gTimeline.mark("startup complete");

        uilayout_onloadDelayed(); // if closed fullscreen, maximize
        RunOutput_Init();



        // initialize the js dialog proxy with our window so that we can
        // avoid using the hidden window
        try {
            var dialogproxy = Components.classes['@activestate.com/asDialogProxy;1'].
                                getService(Components.interfaces.asIDialogProxy);
            dialogproxy.init(window);
        } catch (e) {
            log.exception(e);
        }






        // Openning the Start Page should be before commandment system init and
        // workspace restoration because it should be the first view opened.
        if (gPrefs.getBooleanPref("show_start_page")) {
            open_openStartPage();
        }

        /* commandmentSvc needs the window to be alive */
        _gCommandmentSvc = Components.classes["@activestate.com/koCommandmentService;1"]
                           .getService(Components.interfaces.koICommandmentService);
        _gCommandmentSvc.initialize();





        // the offer to restore the workspace needs to be after the
        // commandments system is initialized because the commandments mechanism
        // is how the determination of 'running in non-interactive mode' happens,
        // which the restoration step needs to know about.
        view_restoreWorkspace();

        // handle window.arguments spec list
        if (window.arguments && window.arguments[0]) {
            var urllist = window.arguments[0].split('|'); //see asCommandLineHandler.js
            for (var i in urllist) {
                open_openURI(urllist[i]);
            }
        }
        
        Toolbox_updateSharedToolbox();

        // Possibly show the "Tip of the Day" dialog.
        if (gPrefs.getBooleanPref("tipShowAtStartup")) {
           launch_TipOfTheDay();
        }

        gEditorTooltipHandler = new tooltipHandler('editorTooltip');
        doContextMenuHackForPlugin();




        // Initialize the Code Intel system *after* startup files are opened
        // via workspace restoration and 'open' commandments. See
        // _CodeIntelObserver.observe() for why.
        CodeIntel_Initialize();
        KomodoRegisterOnUnloadHandler(KomodoUnLoadDelayItems);
        KomodoRegisterOnUnloadHandler(Komodo_CloseSubwindows);
        gPeMacro.hookOnStartup();
        uilayout_RestoreTabSelections();









        // xul crash hack, see http://bugs.activestate.com/show_bug.cgi?id=30774
        // also see toolbar.xml bindings
        window.setTimeout("document.getElementById('toolbox_main').init();",10);
        window.setTimeout("window.updateCommands('interactiveshell');",1);
    } catch(ex) {
        log.exception(ex);
    }
}

function doContextMenuHackForPlugin() {
    // Verified still necessary with moz 1.8 branch - SMC
    // For some reason popups over the plugin are messed up until
    // the first context menu over mozilla is activated. It is apparently
    // due to popupNode not being initialized, so we do that here. See:
    //   http://www.xulplanet.com/references/elemref/ref_popup.html
    try {
        var toolbar = document.getElementById('toolbox_main');
        if (!toolbar) return;
        document.popupNode = toolbar;
    } catch(e) {
        log.exception(e);
    }
}


function KomodoUnLoadDelayItems() {
    uilayout_SaveTabSelections();

    var finalizeSvc = Components.classes["@activestate.com/koFinalizeService;1"]
                  .getService(Components.interfaces.koIFinalizeService);
    finalizeSvc.finalize();
    CodeIntel_Finalize();
    _gCommandmentSvc.finalize();
}


// All Komodo windows should be closed when the main Komodo window is closed.
// This is handled automatically for dialogs opened with the "dependent" or
// "dependent=yes" window feature, e.g.:
//      window.openDialog("chrome://.../foo.xul", "blah", "...dependent...");
// However some Komodo windows cannot use this window feature because it also
// implies that the window is always on top of the main Komodo window. Those
// windows are close by this unload handler.
function Komodo_CloseSubwindows() {
    log.info("Komodo_CloseSubwindows");
    try {
        // Names of windows to close.
        var names = ["komodo_help", "komodo_rxx",
                     "komodo_objectbrowser",
                     "komodo_buildcidb", "komodo-buffer"
                     ];

        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                           .getService(Components.interfaces.nsIWindowMediator);
        for (var i = 0; i < names.length; ++i) {
            //XXX This is presuming that there is only ever zero or one
            //    instances of that window. If that is ever not true then
            //    use .getEnumerator() .getXULEnumerator() instead.
            var name = names[i]
            var w = wm.getMostRecentWindow(name);
            if (w) {
                log.info("Komodo_CloseSubwindows: close '"+name+"' window");
                w.close();
            }
        }
    } catch(ex) {
        log.error("Komodo_CloseSubwindows: "+ex);
    }
}

/*
    if you want to do something onUnload or onClose, then
    you have to register a handler function that takes no
    arguments.  This is called automaticaly.
    onOnload will call the handlers LIFO to reverse the
    order of initialization.
*/
var _gOnUnloadHandlers = [];
function KomodoOnUnload() {
    /* no further changes here, use registration functions below!

      we call onunload handlers LIFO
    */
    while (_gOnUnloadHandlers.length > 0) {
        var callback = _gOnUnloadHandlers.pop();
        try {
            if (callback.object) {
                callback.handler.apply(callback.object);
            } else {
                callback.handler();
            }
        } catch(e) {
            log.exception(e,"error when running '"+callback.handler+
                      "' shutdown handler (object='"+callback.object+"':");
        }
    }

    log.debug("made it to the end of komodoOnUnload\n");
}

function KomodoRegisterOnUnloadHandler(handler, object /*=null*/) {
    // Register a routine to be called on Komodo shutdown.
    // To register a simple routine do this:
    //      KomodoRegisterOnUnloadHandler(<routine>)
    // To register an object method do this:
    //      KomodoRegisterOnUnloadHandler(this.<method>, this);
    //XXX Do we want to add an optional argument list to pass in?
    if (typeof(object) == "undefined") object = null;
    var callback = new Object();
    callback.handler = handler;
    callback.object = object;
    _gOnUnloadHandlers.push(callback);
}


/**
 * KomodoCanClose
 *
 * this function checks with the projectmanager, and the viewmanagers
 * to see if it's ok to shutdown
 *
 * Any services that must access the UI after CanClose, but prior
 * to unload MUST use a PostCanClose handler rather than a CanClose
 * handler.  One example is the autoSave thread in the document
 * service.  This thread accesses scintilla, which is gone during
 * the unload, and unfortunately XPCOM is not handling this correctly
 * and we end up with a thread locked into memory.
 */
var _gCanCloseHandlers = [];
var _gPostCanCloseHandlers = [];
function KomodoCanClose()
{
    try {
        if (gPeMacro.hookOnQuit()) {
            StatusBar_AddMessage("Macro interrupted shutdown procedure.",
                                 "macro",
                                 5000,
                                 true);
            return false;
        }
    } catch (e) {
        log.exception(e);
    }
    var retval;
    /* no further changes here, use registration functions below! */
    for (var i=_gCanCloseHandlers.length-1; i >= 0 ; i--) {
        var callback = _gCanCloseHandlers[i];
        try {
            if (callback.object) {
                retval = callback.handler.apply(callback.object);
            } else {
                retval = callback.handler();
            }
            if (!retval) return false;
        } catch(e) {
            log.exception(e,"error when running '"+callback.handler+
                        "' canclose handler (object='"+callback.object+"':");
        }
    }
    // ok, we CAN close, do the "Post" can close handlers now
    KomodoPostCanClose();
    return true;
}

function KomodoPostCanClose()
{
    /* no further changes here, use registration functions below! */
    for (var i=_gPostCanCloseHandlers.length-1; i >= 0 ; i--) {
        var callback = _gPostCanCloseHandlers[i];
        try {
            if (callback.object) {
                callback.handler.apply(callback.object);
            } else {
                callback.handler();
            }
        } catch(e) {
            log.exception(e,"error when running '"+callback.handler+
                        "' postcanclose handler (object='"+callback.object+"':");
        }
    }
    return;
}

function KomodoRegisterCanCloseHandler(handler, object /* =null */) {
    // Register a routine to be called on Komodo shutdown.
    // and that can stop the quit behavior
    // To register a simple routine do this:
    //      KomodoRegisterCanCloseHandler(<routine>)
    // To register an object method do this:
    //      KomodoRegisterCanCloseHandler(this.<method>, this);
    //XXX Do we want to add an optional argument list to pass in?
    if (typeof(object) == "undefined") object = null;
    var callback = new Object();
    callback.handler = handler;
    callback.object = object;
    _gCanCloseHandlers.push(callback);
}

function KomodoRegisterPostCanCloseHandler(handler, object /* =null */) {
    // See documentation in KomodoRegisterCanCloseHandler above
    // postCanClose handlers are necessary to shutdown some services
    // prior to UI disappearing, but after CanClose succeeds.  Some
    // services fail to shutdown properly during the actual unload
    // handler, namely the autosave thread where we are accessing
    // some XPCOM UI components (scintilla)
    if (typeof(object) == "undefined") object = null;
    var callback = new Object();
    callback.handler = handler;
    callback.object = object;
    _gPostCanCloseHandlers.push(callback);
}

//XXX Mozilla's goQuitApplication() sends a "quit-application"
//XXX notification before tryToClose().  This notify causes nsPluginHostImpl
//XXX to unload all plugins, including Scintilla.  We call ProjectViewerCanClose
//XXX ourselves before passing control on to Moz.
var orig_goQuitApplication = window.goQuitApplication;

function KomodoGoQuitApplication() {
    if (!KomodoCanClose()) {
        return false;
    }
    return orig_goQuitApplication();
}

window.goQuitApplication = KomodoGoQuitApplication;

function NIY(e) {
    dump(e);
    alert("Not implemented yet");
}

