/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFile();

function peFile() {
    this.name = 'peFile';
    this.koLanguageRegistryService =
                Components.classes["@activestate.com/koLanguageRegistryService;1"].
                            getService(Components.interfaces.koILanguageRegistryService);
    this.sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                    getService(Components.interfaces.koISysUtils);
    this.lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                        getService(Components.interfaces.koILastErrorService);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFile.prototype.constructor = peFile;

peFile.prototype.init = function() {
    // register our command handlers
    extensionManager.setDatapoint('Date','lastModifiedTime');
    extensionManager.setDatapoint('Size','fileSize');
    this._clipboard = [];
}

peFile.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_openInDir', this);
    extensionManager.registerCommand('cmd_dumpPartProperties', this);
    extensionManager.registerCommand('cmd_openFilePart', this);
    extensionManager.registerCommand('cmd_makeDirectoryShortcutFromFile', this);
    extensionManager.registerCommand('cmd_refreshStatus', this);
    extensionManager.registerCommand('cmd_editProperties', this);
    extensionManager.registerCommand('cmd_showUnsavedChanges', this);
    extensionManager.registerCommand('cmd_compareFiles', this);
    extensionManager.registerCommand('cmd_compareFileWith', this);
    extensionManager.registerCommand("cmd_exportItems",this);
    extensionManager.registerCommand("cmd_exportToPackage",this);
    extensionManager.registerCommand("cmd_cut",this);
    extensionManager.registerCommand("cmd_copy",this);
    extensionManager.registerCommand("cmd_paste",this);
    extensionManager.registerCommand("cmd_showInFinder",this);
    extensionManager.registerCommand("cmd_renameFile",this);
}

peFile.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_file,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIDirectoryShortcut,'ondblclick',this);
    // we don't add a dblclick handler for Components.interfaces.koIPart_ProjectRef, since
    // it inherits from koIPart_file
}

peFile.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Open File','cmd_openFilePart',
                                    null,
                                    null,
                                    true);
    extensionManager.createMenuItem(Components.interfaces.koIDirectoryShortcut,
                                    'Open...',
                                    'cmd_openInDir',
                                    null,
                                    null,
                                    true /* primary */);
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Make "Open..." Shortcut','cmd_makeDirectoryShortcutFromFile');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Refresh Status','cmd_refreshStatus');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Show unsaved changes','cmd_showUnsavedChanges');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Compare Files...','cmd_compareFiles');







    extensionManager.createMenuItem(Components.interfaces.koIPart,
                                    'Show In File Manager','cmd_showInFinder');

    extensionManager.createMenuItem(Components.interfaces.koIPart,
                                    'Rename...','cmd_renameFile');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Compare File With...','cmd_compareFileWith');
    extensionManager.createMenuItem(Components.interfaces.koIPart,
                                    'Export as Project File...','cmd_exportItems');
    extensionManager.createMenuItem(Components.interfaces.koIPart,
                                    'Export Package...','cmd_exportToPackage');

    // XXX debug info
    extensionManager.createMenuItem(Components.interfaces.koIPart,'Dump',
                                    'cmd_dumpPartProperties');
}

peFile.prototype.supportsCommand = function(command, item) {
    var items = null;
    var file = null;
    if (gFocusedProjectView) {
        items = gFocusedProjectView.getSelectedItems();
    }
    switch (command) {
    case 'cmd_cut':
    case 'cmd_copy':
        if (!items || items.length == 0) return false;
        if (command == 'cmd_cut' && !gFocusedProjectView.manager.writeable()) return false;
        // We want to make sure that no projects are selected
        var notlive = command == 'cmd_cut';
        for (var i = 0; i < items.length; i++) {
            if ((notlive && items[i].live) || items[i].type == 'project') {
                return false;
            }
        }
        return true;
    case 'cmd_paste':
        // XXX we might not want to allow files to be pasted into a live folder
        //if (!items || items.length == 0) return false;
        //for (var i = 0; i < items.length; i++) {
        //    if (items[i].live) {
        //        return false;
        //    }
        //}
        return 1; // see http://bugs.activestate.com/Komodo/show_bug.cgi?id=27527

    case 'cmd_dumpPartProperties':
        // Only return true in developer builds
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                  getService(Components.interfaces.koIInfoService);
        return infoSvc.buildFlavour == "dev";
    case 'cmd_openFilePart':
        for (i = 0; i < items.length; i++) {
            if (items[i].type == 'file') return true;
        }
        return false;
    case 'cmd_showInFinder':
        if (items.length != 1) return false;
        file = items[0].getFile();
        return (items.length == 1 && file && file.isLocal);
    case 'cmd_renameFile':
        if (items.length != 1) return false;
        file = items[0].getFile();
        return (items.length == 1 && items[0].type != 'project' && file && file.isLocal);
    case 'cmd_openInDir':
        return (items.length == 1 && items[0].type == 'DirectoryShortcut');
    case 'cmd_makeDirectoryShortcutFromFile':
        if (! gFocusedProjectView.manager.writeable()) return false;
        return (items.length == 1 && !items[0].live && items[0].type == 'file' && items[0].url.indexOf('file://') == 0);
    case 'cmd_refreshStatus':
        // if a toolbox has focus, get the currently selected item and refresh it
        // otherwise, always refresh the currentView if there is one.
        if (view_getFocusedProjectView()) {
            item = gFocusedProjectView.getSelectedItem();
            // item must be a file
            if (item && item.url) return true;
        }
        return gViewMgr.currentView != null;
        break;
    case 'cmd_editProperties':
        if (!items || items.length < 1) return false;
        for (i = 0; i < items.length; i++) {
            type = items[i].type;
            switch (type) {
                case 'file':
                case 'folder':
                case 'livefolder':
                case 'project':
                case 'DirectoryShortcut':
                case 'URL':
                case 'command':
                case 'macro':
                case 'snippet':
                case 'template':
                case 'webservice':
                    // leave in so we have support for old project files
                case 'menu':
                case 'toolbar':
                    continue;
                default:
                    //dump('returning false because type is not correct\n');
                    return false;
            }
        }
        return true;
    case 'cmd_showUnsavedChanges':
        var view = gViewMgr.currentView;
        return (view && view.document && view.document.isDirty &&
                !view.document.isUntitled);
    case 'cmd_compareFiles':
        return (items.length == 2 && items[0].type == 'file' && items[1].type == 'file');
    case 'cmd_compareFileWith':
        return (items && items.length == 1 && items[0].type == 'file');
    case 'cmd_exportItems':
        // the following isn't great, but it deals with
        // http://bugs.activestate.com/Komodo/show_bug.cgi?id=23226
        // with minimal risk.
        if (items && items.length == 1 && items[0].type == 'project') return 0;
        return items.length > 0;
    case 'cmd_exportToPackage':
        return items && items.length > 0;
    default:
        break;
    }
    return false;
}

peFile.prototype.isCommandEnabled = peFile.prototype.supportsCommand;

function filename_implies_move(name) {
    if (name.indexOf('/') != -1) return true;
    if (name.indexOf('\\') != -1) return true;
    if (name.indexOf('..') != -1) return true;
    return false;
}

peFile.prototype.doCommand = function(command) {
    var item, items = null;
    var i, fname, otherfile;
    var dirname = '';
    if (gFocusedProjectView) {
        item = gFocusedProjectView.getSelectedItem();
        items = gFocusedProjectView.getSelectedItems();
    }
    switch (command) {
    case 'cmd_openFilePart':
        // get the current selection, then open the file
        if (!items) return;
        var paths = [];
        for (i = 0; i < items.length; i++) {
            if (items[i].type == 'file') paths.push(items[i].url);
        }
        open_openMultipleURIs(paths);
        break;
    case 'cmd_showInFinder':
        var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                    getService(Components.interfaces.koISysUtils);
        sysUtilsSvc.ShowFileInFileManager(item.getFile().path)
        break;
    case 'cmd_renameFile':
        var newname;
        while (true) {
            newname = dialog_prompt(
                "Enter a new filename.", // prompt
                null, // label
                item.name, // default
                "Rename File or Folder"); // title
            if (!newname) return; // cancel was hit
            if (!filename_implies_move(newname)) break;
            dialog_alert("The file can be renamed in place, but not moved to a new directory.");
        }
        var osSvc = Components.classes["@activestate.com/koOs;1"]
                                    .getService(Components.interfaces.koIOs);
        file = item.getFile();
        var newfile = osSvc.path.join(item.getFile().dirName, newname);
        osSvc.rename(item.getFile().path, newfile);
        if (!item.live)
            item.url = newfile;
        gFileStatusSvc.getStatusForUri(newfile, true /* forcerefresh */);
        break;
    case 'cmd_makeDirectoryShortcutFromFile':
        var view, url;
        dirname = null;
        if (typeof(item) != 'undefined' && item) {
            url = item.getStringAttribute('url');
            var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                        .getService(Components.interfaces.koIOsPath);
            dirname = uriparse_URIToLocalPath(url);
            if (item.type != 'folder') {
                dirname = pathSvc.dirname(dirname);
            }
        }
        peFile_addDirectoryShortcut(dirname, item);
        break;
    case 'cmd_refreshStatus':
        doRefreshStatus();
        break;
    case 'cmd_dumpPartProperties':
        if (!gFocusedProjectView) return;
        item = gFocusedProjectView.getSelectedItem();
        if (item) item.dump(0);
        break;
    case 'cmd_openInDir':
        if (!gFocusedProjectView) return;
        item = gFocusedProjectView.getSelectedItem();
        if (item) {
            OpenDirectoryShortcut(item)
        }
        break;
    case 'cmd_editProperties':
        //if (!view_getFocusedProjectView())
        //    return;
        if (!items) return;
        for (i = 0; i < items.length; i++) {
            item = items[i];
            switch (item.type) {
                case 'webservice':
                    // leave in so we have support for old project files
                case 'URL':
                    peURL_editProperties(item, i);
                    continue;
                case 'command':
                    command_editProperties(item, i);
                    continue;
                case 'macro':
                    macro_editProperties(item, i);
                    continue;
                case 'snippet':
                    snippet_editProperties(item, i);
                    continue;
                case 'DirectoryShortcut':
                    this.editDirectoryShortcut(item, i);
                    continue;
                case 'template':
                case 'file':
                    peFile_Properties(item, null, false);
                    continue;
                case 'folder':
                case 'livefolder':
                case 'project':
                    peFile_Properties(item, null, true);
                    continue;
                case 'menu':
                case 'toolbar':
                    peMenu_editProperties(item, i);
                    continue;
            }
        }
        break;
    case 'cmd_showUnsavedChanges':
        view = gViewMgr.currentView;
        var changes = view.document.getUnsavedChanges();
        launch_DiffWindow(changes,
                          "unsaved changes: "+view.document.displayPath);
        break;
    case 'cmd_compareFiles':
        if (!gFocusedProjectView || !items || items.length == 0) {
            return;
        } else {
            fname = items[0].getFile().URI;
            otherfile = items[1].getFile().URI;
        }
        peFile_ShowDiffs(fname, otherfile);
        return;
    case 'cmd_compareFileWith':
        if (!gFocusedProjectView) return;
        var file = items[0].getFile()
        fname = file.URI;
        var pickerDir = file.isLocal? file.dirName : '';
        otherfile = filepicker_openFile(pickerDir);
        if (otherfile) {
            peFile_ShowDiffs(fname, otherfile);
        }
        return;
    case "cmd_exportItems":
        if (!gFocusedProjectView) return;
        Toolbox_ExportItems(items);
        break;
    case 'cmd_exportToPackage':
        Toolbox_ExportPackageItems(items);
        break;
    case 'cmd_cut':
        // get the current selection, then open the file
        if (!gFocusedProjectView) return;
        this.doCut(items)
        break;
    case 'cmd_copy':
        // get the current selection, then open the file
        this.doCopy(items)
        break;
    case 'cmd_paste':
        if (this._clipboard.length == 0) return;
        this.doPaste(items)
        break;
    default:
        break;
    }
}

function peFile_addDirectoryShortcut(dirname, /*koIPart*/ parent)
{
    if (typeof(parent) == 'undefined' || !parent)
        parent = gFocusedProjectView.manager.getCurrentProject();
    try {
        var dirshortcut = parent.project.createPartFromType('DirectoryShortcut');
        dirshortcut.value = dirname;
        if (dirname.length > 25) {
            dirname = dirname.substr(0, 10) + '...' + dirname.substr(dirname.length - 15, dirname.length)
        }
        dirshortcut.setStringAttribute('name', dirname);
        gFocusedProjectView.manager.addItem(dirshortcut, parent);
    } catch(e) {
        log.exception(e);
    }
}

function _openDiffWindowForFiles(fname1, fname2) {
    window.setCursor("wait");
    try {
        var diff = Components.classes['@activestate.com/koDiff;1']
                      .createInstance(Components.interfaces.koIDiff);
        try {
            diff.initByDiffingFiles(fname1, fname2);
        } catch (ex) {
            log.error(ex);
            var msg = "Error calculating differences: " +
                      this.lastErrorSvc.getLastErrorMessage();
            alert(msg);
            return;
        }

        if (!diff.diff) {
            var txt = "There are no differences between '"+fname1+
                      "' and '"+fname2+"'.";
            if (diff.doc1.existing_line_endings !=
                diff.doc2.existing_line_endings) {
                txt = "The line endings in the files are different, but there "+
                      "are no other differences between the files.";
            }
            dialog_alert(txt);
        } else {
            launch_DiffWindow(diff.diff,
                              "compare files: "+uriparse_baseName(fname1)
                                +", "+uriparse_baseName(fname2),
                              diff.warning);
        }
    } finally {
        window.setCursor('auto');
    }
}


peFile.prototype.ondblclick = function(item,event) {
    if (item.type == 'DirectoryShortcut') {
        OpenDirectoryShortcut(item);
    } else {
        open_openURI(item.url);
    }
}

function doRefreshStatus () {
    var urls = {};
    var url;
    var item, items;

    if (view_getFocusedProjectView()) {
        // gProjectManager.log.debug("project or toolbox has focus\n");
        items = gFocusedProjectView.getSelectedItems();
        // otherwise, look at the focused project view's selected parts.
        // XXX optimize this!
        for (i=0; i<items.length; i++) {
            urls[items[i].url]=1;
        }
    } else {
        // always fall back to the current view
        var view = gViewMgr.currentView;
        item = view.item;
        if (!view.document || view.document.isUntitled) {
            // This is an unsaved file -- cannot refresh.
            return;
        }
        item.url = view.document.file.URI;
        urls[item.url]=1;

        // Also refresh CodeIntel data for this view, if enabled.
        if (view.isCICitadelStuffEnabled || view.isCIXMLStuffEnabled) {
            // fake an edit to get a re-scan of the document
            gCodeIntelSvc.ideEvent_EditedCurrentDocument(
                view.document,
                view.scimoz,
                0, // no lines added
                true); // yes, rescan
        }
    }
    try {
        for (url in urls) {
            // gProjectManager.log.debug("updating status for url: "+url+"\n");
            gFileStatusSvc.getStatusForUri(url, true /* forcerefresh */);
        }
    } catch(e) {
        gProjectManager.log.error(e);
    }
}

function peFile_Properties(item, view, folder)
{
    if (typeof(view) == 'undefined') view = null;
    if (typeof(folder) == 'undefined') folder = false;
    if (typeof(item) == 'undefined') item = null;

    var file = null;
    if (item && !view)
        file = item.getFile();
    if (file) {
        // try to get the view from the view manager
        view = gViewMgr.getViewForFile(file.URI);
    }
    try {
        // Handle cancel from prefs window
        var resp = new Object ();
        resp.res = "";
        resp.part = item;
        resp.title = "File Properties and Settings";
        resp.folder = folder;
        resp.view = view;
        if (item && item.type == "project") {
            resp.title = "Project Properties and Settings";
        } else if (folder) {
            resp.title = "Folder Properties and Settings";
        }
        try {
            openWindowMultipleInstance(
                    "chrome://komodo/content/pref/project.xul",
                    'komodo_projectprefs',
                    "chrome,dialog=no,dependent,resizable,close=yes,modal=yes",
                    resp);
        } catch(ex) {
            log.exception(ex);
            //log.warn("error opening preferences dialog:"+ex);
            return false;
        }
        if (resp.res != "ok") {
            return false;
        }
        // make the tree refresh this part
        gFocusedProjectView.view.refresh(item);
        return true;
    } catch (e) {
        log.exception(e);
    }
    return false;
}

peFile.prototype.editDirectoryShortcut = function(item) {
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    obj.imgsrc = 'chrome://komodo/skin/images/open.png';
    obj.type = 'DirectoryShortcut';
    obj.prettytype = 'Directory Shortcut';
    openWindowMultipleInstance(
        "chrome://komodo/content/project/simplePartProperties.xul",
        "komodo_DirectoryShortcutProperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

peFile.prototype.doCut = function(items)
{
    var needUpdate = this._clipboard.length == 0 && items.length != 0;
    this._clipboard = [];
    var i, index, parent;
    // Find out what the "first" index of the parts is
    var first_index = null;
    var parentindex = 0;
    for (i = 0; i < items.length; i++) {
        index = gFocusedProjectView.getIndexByPart(items[i]);
        if (first_index == null || index < first_index) {
            first_index = index;
            parentindex = gFocusedProjectView.getIndexByPart(items[i].parent);
        }
    }

    // Do the cut
    for (i = 0; i < items.length; i++) {
        this._clipboard.push(items[i].clone());
    }
    gFocusedProjectView.manager.removeItems(items, false);
    // Select the parent
    if (parentindex != -1) {
        gFocusedProjectView.selection.select(parentindex);
    }
    if (needUpdate) {
        window.updateCommands('clipboard');
    }
}

peFile.prototype.doCopy = function(items)
{
    // XXX use moz clipboard for this?
    var needUpdate = this._clipboard.length == 0 && items.length != 0;
    this._clipboard = [];
    for (var i = 0; i < items.length; i++) {
        this._clipboard.push(items[i].clone());
    }
    if (needUpdate) {
        window.updateCommands('clipboard');
    }
}

peFile.prototype.doPaste = function()
{
    // XXX use moz clipboard for this?
    for (var i = 0; i < this._clipboard.length; i++) {
        item = this._clipboard[i];
        if (item) {
            gFocusedProjectView.manager.addItem(item.clone(), gFocusedProjectView.getSelectedItem());
        }
    }
}

function OpenDirectoryShortcut(part) {
    var defaultDirectory = part.value;
    var paths = filepicker_openFiles(defaultDirectory);
    if (paths == null)
        return;
    open_openMultipleURIs(paths);
}

function peFile_CompareTwoFiles()
{
    // We've been picked from the tools menu
    fname = filepicker_openFile(null, null, "Select first file");
    if (!fname) return;
    otherfile = filepicker_openFile(null, null, "Select second file");
    if (!otherfile) return;
    peFile_ShowDiffs(fname, otherfile);
}

function peFile_ShowDiffs(fname1, fname2) {
    // fname1/2 can be filenames or uri's.
    fname1 = fname1.replace("\\", "\\\\", 'g');
    fname2 = fname2.replace("\\", "\\\\", 'g');
    // difflib can be slow, so we do this in a timeout
    window.setTimeout("_openDiffWindowForFiles('"+fname1+"', '"+fname2+"');",
                      0);
}

